class Server:
    def __init__(self, attr: dict, panel):
        self._panel = panel

        self.id = attr.get("identifier")
        self.uuid = attr.get("uuid")
        self.name = attr.get("name")
        self.owner = attr.get("server_owner")
        self.node = attr.get("node")
        self.description = attr.get("description", "")
        self.invocation = attr.get("invocation", "")
        self.status = attr.get("status")
        self.sftp_ip = attr.get("sftp_details", {}).get("ip")
        self.sftp_port = attr.get("sftp_details", {}).get("port")
        
        limits = attr.get("limits", {})
        self.memory = limits.get("memory")
        self.disk = limits.get("disk")
        self.cpu = limits.get("cpu")

        relationships = attr.get("relationships", {})
        self.allocations = [
            f'{a["attributes"]["ip"]}:{a["attributes"]["port"]}'
            for a in relationships.get("allocations", {}).get("data", [])
        ]

    def __repr__(self):
        return f"<Server name={self.name}, owner={self.owner}, node={self.node}>"
    
    def _power_action(self, signal: str):
        return self._panel._request(
            "POST",
            f"servers/{self.id}/power",
            json={"signal": signal}
        )
    
    def start(self):
        return self._power_action("start")

    def stop(self):
        return self._power_action("stop")

    def restart(self):
        return self._power_action("restart")
    
    def send_command(self, cmd: str):
        return self._panel._request(
            "POST",
            f"servers/{self.id}/command",
            json={"command": cmd}
        )
    
    def add_subuser(self, email: str, permissions: list):
        permission_keys = [perm["key"] for perm in permissions]
        return self._panel._request(
            "POST",
            f"servers/{self.id}/users",
            json={
                "email": email, 
                "permissions": [
                    "control.start",
                    "control.stop",
                    "control.restart",
                    "control.console"
                ]
            }
        ); print("Successfully added a subuser!")
    
    def list_subusers(self):
        return self._panel._request(
            "GET",
            f"servers/{self.id}/users",
        )
    
    def delete_subuser(self, username: str):
        return self._panel._request(
            "DELETE",
            f"servers/{self.id}/users/{username}"
        )
    
    def rename(self, name: str, description: str):

        data = {"name": name}
        if description is not None:
            data["description"] = description

        return self._panel._request(
            "POST",
            f"servers/{self.id}/settings/rename",
            json=data
        )
    
    def reinstall(self):
        return self._panel._request(
            "POST",
            f"servers/{self.id}/settings/reinstall"
        )
    



from setuptools import setup, find_packages

setup(
    name="pelipy",
    version="1.0.0",
    description="A Python wrapper for the Pelican Panel",
    long_description=open("README.md", encoding="utf-8").read(),
    long_description_content_type="text/markdown",  # Fixed missing comma
    author="uhalexz",
    url="https://github.com/uhalexz/pelipy",
    packages=find_packages(),
    install_requires=["requests"],
    python_requires=">=3.7",
    classifiers=[
        "Programming Language :: Python :: 3",  # Fixed typo
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    license="MIT",
    keywords="pelican panel api wrapper",
    project_urls={
        "Bug Tracker": "https://github.com/uhalexz/pelipy/issues",
        "Documentation": "https://github.com/uhalexz/pelipy#readme",
        "Source Code": "https://github.com/uhalexz/pelipy",
    },
)


import requests
from .objects import *

class Pelican:
    def __init__(self, api_key: str, panel_url: str):
        self.api_key = api_key
        self.api_url = panel_url + "/api/client/"
        self.headers = {
            "Authorization": f"Bearer {api_key}",
            "Accept": "application/json"
        }

        print(self.api_url)
        print(self.headers)
        print(self.api_key)
    
    def _request(self, method: str, endpoint: str, **kwargs):
        url = f"{self.api_url}{endpoint}"
        response = requests.request(method, url, headers=self.headers, **kwargs)

        if response.status_code == 204:
            return None

        try:
            data = response.json()
        except requests.JSONDecodeError:
            raise Exception(
                f"Failed to decode JSON:\nStatus: {response.status_code}\nURL: {url}\nResponse: {response.text[:300]}"
            )

        if response.status_code != 200:
            raise Exception(f"API Error: {response.status_code} - {response.text}")

        return data

    
    def server(self, server_id: str) -> Server:
        data = self._request("GET", f"servers/{server_id}")
        return Server(data.get("attributes", {}), self)


class Permissions:
    """Defines available permissions for subusers with descriptions."""

    # Control Permissions
    console = {"key": "control.console"}
    start = {"key": "control.start"}
    stop = {"key": "control.stop"}
    restart = {"key": "control.restart"}
    
    # User Permissions
    create_user = {"key": "user.create"}
    read_user = {"key": "user.read"}
    update_user = {"key": "user.update"}
    delete_user = {"key": "user.delete"}

    # File Permissions
    create_file = {"key": "file.create"}
    read_file = {"key": "file.read"}
    readcontent_file = {"key": "file.read-content"}
    file_update = {"key": "file.update"}
    file_delete = {"key": "file.delete"}
    file_archive = {"key": "file.archive"}
    sftp = {"key": "file.sftp"}

    # Backup Permissions
    backup_create = {"key": "backup.create"}
    backup_read = {"key": "backup.read"}
    backup_delete = {"key": "backup.delete"}
    backup_download = {"key": "backup.download"}
    backup_restore = {"key": "backup.restore"}

    # Allocation Permissions
    alloc_read = {"key": "allocation.read"}
    alloc_create = {"key": "allocation.create"}
    alloc_update = {"key": "allocation.update"}
    alloc_delete = {"key": "allocation.delete"}

    # Startup Permissions
    startup_read = {"key": "startup.read"}
    startup_update = {"key": "startup.update"}
    startup_docker = {"key": "startup.docker-image"}

    # Database Permissions
    db_create = {"key": "database.create"}
    db_read = {"key": "database.read"}
    db_update = {"key": "database.update"}
    db_delete = {"key": "database.delete"}
    db_viewpwd = {"key": "database.view_password"}

    # Schedule Permissions
    schedule_create = {"key": "schedule.create"}
    schedule_read = {"key": "schedule.read"}
    schedule_update = {"key": "schedule.update"}
    schedule_delete = {"key": "schedule.delete"}

    # Settings Permissions
    settings_rename = {"key": "settings.rename"}
    settings_reinstall = {"key": "settings.reinstall"}

    # Activity Permissions
    activity_read = {"key": "activity.read"}